@@ -29,6 +29,7 @@
 #include <X11/extensions/XInput.h>
 #include <X11/Xatom.h>
 #include <string.h>
+#include "gpds-xinput-utils.h"
 
 typedef struct _GpdsXInputPriv GpdsXInputPriv;
 struct _GpdsXInputPriv
@@ -147,12 +148,6 @@ get_property (GObject    *object,
     }
 }
 
-GQuark
-gpds_xinput_error_quark (void)
-{
-    return g_quark_from_static_string("gpds-xinput-error-quark");
-}
-
 GpdsXInput *
 gpds_xinput_new (const gchar *device_name)
 {
@@ -163,53 +158,12 @@ gpds_xinput_new (const gchar *device_name)
                         NULL);
 }
 
-static XDeviceInfo *
-get_device_info (const gchar *device_name)
+const gchar *
+gpds_xinput_get_device_name (GpdsXInput *xinput)
 {
-    XDeviceInfo *device_infos;
-    gint i, n_device_infos;
+    g_return_val_if_fail(GPDS_IS_XINPUT(xinput), NULL);
 
-    device_infos = XListInputDevices(GDK_DISPLAY(), &n_device_infos);
-
-    for (i = 0; i < n_device_infos; i++) {
-        if (device_infos[i].use != IsXExtensionPointer)
-            continue;
-        if (!strcmp(device_infos[i].name, device_name))
-            return &device_infos[i];
-    }
-
-    XFreeDeviceList(device_infos);
-
-    return NULL;
-}
-
-static XDevice *
-open_device (const gchar *device_name, GError **error)
-{
-    XDeviceInfo *device_info;
-    XDevice *device;
-
-    device_info = get_device_info(device_name);
-    if (!device_info) {
-        g_set_error(error,
-                    GPDS_XINPUT_ERROR,
-                    GPDS_XINPUT_ERROR_NO_DEVICE,
-                    _("No device found."));
-        return NULL;
-    }
-
-    gdk_error_trap_push();
-    device = XOpenDevice(GDK_DISPLAY(), device_info->id);
-    gdk_error_trap_pop();
-    if (!device) {
-        g_set_error(error,
-                    GPDS_XINPUT_ERROR,
-                    GPDS_XINPUT_ERROR_NO_DEVICE,
-                    _("Could not open %s device."), device_name);
-        return NULL;
-    }
-
-    return device;
+    return GPDS_XINPUT_GET_PRIVATE(xinput)->device_name;
 }
 
 static XDevice *
@@ -220,7 +174,7 @@ get_device (GpdsXInput *xinput, GError **error)
     if (priv->device)
         return priv->device;
 
-    priv->device = open_device(priv->device_name, error);
+    priv->device = gpds_xinput_utils_open_device(priv->device_name, error);
     return priv->device;
 }
 
@@ -243,7 +197,7 @@ gpds_xinput_set_int_properties (GpdsXInput *xinput,
     if (!device)
         return FALSE;
 
-    property_atom = XInternAtom(GDK_DISPLAY(), property_name, False);
+    property_atom = gdk_x11_get_xatom_by_name(property_name);
 
     switch (format_type) {
     case 8:
@@ -299,15 +253,13 @@ get_atom (GpdsXInput *xinput, const gchar *property_name, GError **error)
 
     properties = XListDeviceProperties(GDK_DISPLAY(), device, &n_properties);
     for (i = 0; i < n_properties; i++) {
-        gchar *name;
+        const gchar *name;
 
-        name = XGetAtomName(GDK_DISPLAY(), properties[i]);
+        name = gdk_x11_get_xatom_name(properties[i]);
         if (!strcmp(name, property_name)) {
             found_atom = properties[i];
-            XFree(name);
             break;
         }
-        XFree(name);
     }
     XFree(properties);
 
@@ -378,22 +330,6 @@ get_int_properties (GpdsXInput *xinput,
     return TRUE;
 }
 
-static Atom
-get_float_atom (GError **error)
-{
-    Atom float_atom;
-
-    float_atom = XInternAtom(GDK_DISPLAY(), "FLOAT", False);
-    if (float_atom == 0) {
-        g_set_error(error,
-                    GPDS_XINPUT_ERROR,
-                    GPDS_XINPUT_ERROR_NO_FLOAT_ATOM,
-                    _("No float atom in XServer"));
-    }
-
-    return float_atom;
-}
-
 gboolean
 gpds_xinput_get_int_properties (GpdsXInput *xinput,
                                 const gchar *property_name,
@@ -430,11 +366,11 @@ gpds_xinput_set_float_properties (GpdsXInput *xinput,
     if (!device)
         return FALSE;
 
-    float_atom = get_float_atom(error);
+    float_atom = gpds_xinput_utils_get_float_atom(error);
     if (float_atom == 0)
         return FALSE;
 
-    property_atom = XInternAtom(GDK_DISPLAY(), property_name, False);
+    property_atom = gdk_x11_get_xatom_by_name(property_name);
 
     property_data = g_new(gfloat, n_properties);
     for (i = 0; i < n_properties; i++)
@@ -475,7 +411,7 @@ gpds_xinput_get_float_properties (GpdsXInput *xinput,
     if (!device)
         return FALSE;
 
-    float_atom = get_float_atom(error);
+    float_atom = gpds_xinput_utils_get_float_atom(error);
     if (float_atom == 0)
         return FALSE;
 
@@ -511,12 +447,6 @@ gpds_xinput_get_float_properties (GpdsXInput *xinput,
     return TRUE;
 }
 
-gboolean
-gpds_xinput_exist_device (const gchar *device_name)
-{
-    return get_device_info(device_name) ? TRUE : FALSE;
-}
-
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
