@@ -48,8 +48,8 @@ teardown (void)
 void
 test_names (void)
 {
+    expected_names = g_list_append(expected_names, "mouse");
     expected_names = g_list_append(expected_names, "touchpad");
-    expected_names = g_list_append(expected_names, "trackpoint");
 
     names = gpds_uis_get_names();
 
@@ -59,7 +59,7 @@ test_names (void)
 void
 test_new (void)
 {
-    ui = gpds_ui_new("touchpad");
+    ui = gpds_ui_new("touchpad", NULL);
     cut_assert(ui);
 }
 
